//============================================================================
// Name        : oosdlib.cpp
// Author      : G Lipari
// Version     :
// Copyright   : OOSD
// Description : Hello World in C++, Ansi-style
//============================================================================

#include <iostream>
#include <string>
#include "observer.h"
using namespace std;
using namespace OOSD;

class Subject : public BaseObservable {
	int i;
public:
	Subject() : i(0) {}
	void increment() {
		i++;
		notifyAll();
	}
	int get() { return i; }
};

class MyClass: public BaseObservable {
	string name;
public:
	MyClass(const string &n) : name(n) {}
	string getName() { return name; }
	void setName(const string &s) { name = s; notifyAll(); }
};

class MyObserver {
public:
	void update(Subject &s) {
		cout << "MyObserver: " << s.get() << endl;
	}
	void update(MyClass &s) {
		cout << "MyObserver: " << s.getName() << endl;
	}
};

class AnotherObserver {
	MyObserver &obs;
public:
	AnotherObserver(MyObserver &o) : obs(o) {}
	void update(Subject &s) {
		cout << "AnotherObserver " << s.get() << endl;
		cout << "Removing MyObserver from subject" << endl;
		unregister_observer(s, obs);
		cout << "Done" << endl;
	}
	void update(MyClass &s) {
		cout << "AnotherObserver " << s.getName() << endl;
		cout << "Removing myself:" << endl;
		unregister_observer(s, *this);
		cout << "Done" << endl;
	}
};

int main() {
	Subject s;
	MyClass c("pluto");
	MyObserver obs;
	AnotherObserver obs2(obs);

	register_observer(s, obs);
	register_observer(c, obs);
	register_observer(s, obs2);
	register_observer(c, obs2);

	s.increment();
	s.increment();

	c.setName("pippo");

	cout << "End" << endl; // prints
	return 0;
}
